// SuperCollider is an interpreted language - it runs  code as you send it without compiling the whole file at once
// To run code you send it to the SuperCollider sever. Send code by putting your cursor on the line of code you want to send and
// then press Ctrl+Enter (WIN+LINUX) or CMD+Enter (Mac). If you have a "block" of code surrounded by ( )'s you can activate the
// whole block at one time by putting your cursor anywhere in the block and pressing Ctrl+Enter (WIN+LINUX) or CMD+Enter (Mac)

// If things go horribly wrong, try going to the menu bar: Lanugage/Reboot Interpreter...this basically restarts everything.
// You'll have to activate each block of code again - basically starting from scratch without quitting and restarting the program.

// This script is used to analyse your computer's audio input, calculate Mel Frequency Cepstrum Coefficients (MFCC), and then send those out
// as an Open Sound Control message that Wekinator uses as input data for a Neural Net classifier.

// Activate the following blocks of code in order:

// Boot the default sound server - basically get the computer ready to do calculations and receive audio
(
s.boot;
)

// Monitor (i.e. listen to) the default Audio Input of your computer
// This may cause a feedback loop if your input is your microphone and the output is your speakers
// Using headphones is one way to avoide feedback, or you can just skip this step
// If you do get crazy feedback you can stop all audio by pressing
// Ctrl+. (WIN and LINUX) or CMD+. (Mac)
{Out.ar(0,[SoundIn.ar(0), SoundIn.ar(0)])}.play;



// This block of creates an address where you'll send data to. This is the default Wekinator address,
// but you can change it, you just have to also change it later in Wekinator so the messages go to the right place.
(
// create a network address and assign it to a global variable
~toWekinator = NetAddr.new("127.0.0.1", 6448);
)

// If you want you can send a test message - this step is optional
// test sending a message
~toWekinator.sendMsg("/wek/inputs", 0.0, 0.4, 0.8);

// This block of code is used to calculate the data Wekinator will use. It first calculates a Fast Fourier Transform from the audio input,
// then converts that to MFCC values which will send in the next step
(
x = {
    var in, fft, array;
    in = SoundIn.ar(0);
    fft = FFT(LocalBuf(1024), in);  // for sampling rates 44100 and 48000
    array = MFCC.kr(fft);
	Out.kr(0, array); // control bus out
}.play;
)

// Create a data "bus" we'll use in the next block to actually send the data
(
c = Bus(\control, 0, 13);
)

// Start a process "r" that runs until we stop it
// This chunk of code sends 13 MFCC coefficients out as an OSC message in the format Wekinator is looking for.
(
r = {

    inf.do {
		c.getn(13, { |val| ~toWekinator.sendMsg("/wek/inputs",*val[0..]) });
		0.02.wait; // 50 frames per second
    }

}.fork
)

// If you want to stop sending data, you can activate the next line of code
r.stop;