//Necessary for OSC communication with Wekinator:
import oscP5.*;
import netP5.*;
OscP5 oscP5;
NetAddress dest;

// These are needed for the moving average calculation
float[] data1 = new float[2];
float total1 = 0, average1 = 0;
int m1 = 0, n1 = 0;

float[] data2 = new float[2];
float total2 = 0, average2 = 0;
int m2 = 0, n2 = 0;

float[] data3 = new float[2];
float total3 = 0, average3 = 0;
int m3 = 0, n3 = 0;

// Maximum value for the three parameters, somewhere in the range 0-255
int decay1 = 250;
int decay2 = 250;
int decay3 = 250;

// How much the stored value decays each frame - a bigger number is a faster decay
int decayDecreaseAmount = 20;
int decayIncreaseAmount = 100;

// Set thresholds for the ML output from Wekinator
// This helps deal with noisy output from Wekinator
float lowLim = 0.2;
float highLim = 0.8;

void setup()
{
  // Uncomment the line below for full screen mode - press escape to exit fullscreen
  // when it starts.
  fullScreen();
  // Uncomment the line below for a small windowed mode instead of fullscreen
  //size(800, 400);
  //Initialize OSC communication
  oscP5 = new OscP5(this,12000); //listen for OSC messages on port 12000 (Wekinator default)
  dest = new NetAddress("127.0.0.1",6448); //send messages back to Wekinator on port 6448, localhost (this machine) (default)
  rectMode(CENTER);
  
float[] numbers = {1,2,3,4,5,6,7,8,9,10};
  for(int i = 0; i < numbers.length; i++){
    nextValue1(numbers[i]);
    nextValue2(numbers[i]);
    nextValue3(numbers[i]);
    println(average1);
    println(average2);
    println(average3);
  }  
}

void draw()
{
  // If we have the output of Wekinator clearly identify 1,0,0
  if((average1 > highLim) && (average2 < lowLim) && (average3 < lowLim)) {
    decay1 += decayIncreaseAmount;
    decay2 -= decayDecreaseAmount;
    decay3 -= decayDecreaseAmount;
  // If we have the output of Wekinator clearly identify 0,1,0
  } else if((average1 < lowLim) && (average2 > highLim) && (average3 < lowLim)){
    decay1 -= decayDecreaseAmount;
    decay2 += decayIncreaseAmount;
    decay3 -= decayDecreaseAmount;
  // If we have the output of Wekinator clearly identify 0,0,1
  } else if((average1 < lowLim) && (average2 < lowLim) && (average3 > highLim)){
    decay1 -= decayDecreaseAmount;
    decay2 -= decayDecreaseAmount;
    decay3 += decayIncreaseAmount;
  // If we have the output of Wekinator with no clearly identified state
  } else {
    decay1 -= decayDecreaseAmount;
    decay2 -= decayDecreaseAmount;
    decay3 -= decayDecreaseAmount; 
  }
  // Constrain so we have valid RGB color values
  decay1 = constrain(decay1,0,255);
  decay2 = constrain(decay2,0,255);
  decay3 = constrain(decay3,0,255);
    
  //We're going to use these 3 numbers in a simple way
  //Just set the background colour using each value as RGB
  color c = color(decay1,decay2,decay3);
  background(c);
}

//This is called automatically when OSC message is received
void oscEvent(OscMessage theOscMessage) {
  if (theOscMessage.checkAddrPattern("/wek/outputs")==true) {
     if(theOscMessage.checkTypetag("fff")) { //Now looking for parameters
      float p1 = theOscMessage.get(0).floatValue(); //get first parameter
      float p2 = theOscMessage.get(1).floatValue(); //get second parameter
      float p3 = theOscMessage.get(2).floatValue(); //get third parameter
      nextValue1(p1);
      nextValue2(p2);
      nextValue3(p3);        
    } else {
      println("Error: unexpected params type tag received by Processing");
    }
  }
}

// Use averaging to make the ML output from Wekinator a bit smoother
// moving average1
public void nextValue1(float value){
  total1 -= data1[m1];
  data1[m1] = value;
  total1 += value;
  m1 = ++m1 % data1.length;
  if(n1 < data1.length) n1++;
  average1 = total1 / n1;
}

// moving average2
public void nextValue2(float value){
  total2 -= data2[m2];
  data2[m2] = value;
  total2 += value;
  m2 = ++m2 % data2.length;
  if(n2 < data2.length) n2++;
  average2 = total2 / n2;
}

// moving average3
public void nextValue3(float value){
  total3 -= data3[m3];
  data3[m3] = value;
  total3 += value;
  m3 = ++m3 % data3.length;
  if(n3 < data3.length) n3++;
  average3 = total3 / n3;
}
